@@ -42,6 +42,7 @@ THE SOFTWARE.
 #include <libxml/xpointer.h>
 #include <libxml/parserInternals.h>
 #include <libxml/uri.h>
+#include <libexslt/exslt.h>
 
 #include "xmlstar.h"
 
@@ -232,7 +233,8 @@ extract_ns_defs(xmlDocPtr doc, xmlXPathContextPtr ctxt)
     if (!root) return;
 
     for (nsDef = root->nsDef; nsDef; nsDef = nsDef->next) {
-        xmlXPathRegisterNs(ctxt, nsDef->prefix, nsDef->href);
+        if (nsDef->prefix != NULL) /* can only register ns with prefix */
+            xmlXPathRegisterNs(ctxt, nsDef->prefix, nsDef->href);
     }
 }
 
@@ -257,10 +259,15 @@ edUpdate(xmlDocPtr doc, xmlNodeSetPtr nodes, const char *val,
         /* update node */
         if (type == XML_EXPR) {
             xmlXPathObjectPtr res;
+            xmlChar *string;
+
             ctxt->node = nodes->nodeTab[i];
             res = xmlXPathConvertString(xmlXPathCompiledEval(xpath, ctxt));
-            xmlNodeSetContent(nodes->nodeTab[i], res->stringval);
+            /* TODO: do we need xmlEncodeEntitiesReentrant() too/instead? */
+            string = xmlEncodeSpecialChars(doc, res->stringval);
             xmlXPathFreeObject(res);
+            xmlNodeSetContent(nodes->nodeTab[i], string);
+            xmlFree(string);
         } else {
             /* TODO: do we need xmlEncodeEntitiesReentrant() too/instead? */
             xmlChar *content = xmlEncodeSpecialChars(NULL, (const xmlChar*) val);
@@ -373,7 +380,17 @@ edProcess(xmlDocPtr doc, XmlEdAction* ops, int ops_count)
 {
     int k;
     xmlXPathContextPtr ctxt = xmlXPathNewContext(doc);
+    /* NOTE: later registrations override earlier ones */
+#if HAVE_EXSLT_XPATH_REGISTER
+    /* register extension functions */
+    exsltDateXpathCtxtRegister(ctxt, BAD_CAST "date");
+    exsltMathXpathCtxtRegister(ctxt, BAD_CAST "math");
+    exsltSetsXpathCtxtRegister(ctxt, BAD_CAST "set");
+    exsltStrXpathCtxtRegister(ctxt, BAD_CAST "str");
+#endif
+    /* namespaces from doc */
     extract_ns_defs(doc, ctxt);
+    /* namespaces from command line */
     nsarr_xpath_register(ctxt);
     ctxt->node = xmlDocGetRootElement(doc);
 
@@ -455,6 +472,18 @@ edOutput(const char* filename, edOptions g_ops)
 
     edProcess(doc, ops, ops_count);
 
+    /* avoid getting ASCII CRs in UTF-16/UCS-(2,4) text */
+    if ((xmlStrcasestr(doc->encoding, BAD_CAST "UTF") == 0
+            && xmlStrcasestr(doc->encoding, BAD_CAST "16") == 0)
+        ||
+        (xmlStrcasestr(doc->encoding, BAD_CAST "UCS") == 0
+            && (xmlStrcasestr(doc->encoding, BAD_CAST "2") == 0
+                ||
+                xmlStrcasestr(doc->encoding, BAD_CAST "4") == 0)))
+    {
+        set_stdout_binary();
+    }
+
     save = xmlSaveToFilename(g_ops.inplace? filename : "-", NULL, save_options);
     xmlSaveDoc(save, doc);
     xmlSaveClose(save);
