@@ -4,6 +4,14 @@
 #include <config.h>
 #include <stdlib.h>
 
+#if HAVE_SETMODE && HAVE_DECL_O_BINARY
+# include <io.h>
+# include <fcntl.h>
+# define set_stdout_binary() setmode(1, O_BINARY)
+#else
+# define set_stdout_binary()
+#endif
+
 #include <libxml/xmlreader.h>
 
 typedef enum { /* EXIT_SUCCESS = 0, EXIT_FAILURE = 1, */
@@ -12,10 +20,12 @@ typedef enum { /* EXIT_SUCCESS = 0, EXIT_FAILURE = 1, */
 
 #define COUNT_OF(array) (sizeof(array)/sizeof(*array))
 
+typedef enum { QUIET, VERBOSE } Verbosity;
+
 typedef struct _errorInfo {
     const char *filename; /* file error occured in, if any, else NULL */
     xmlTextReaderPtr xmlReader;
-    int verbose;
+    Verbosity verbose;
 } ErrorInfo;
 
 void reportError(void *ptr, xmlErrorPtr error);
